"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const args_1 = require("./args");
const docker_1 = require("./docker");
const ganache_1 = require("./ganache");
const config_1 = require("./config");
const deploy_1 = require("./deploy");
const logger_1 = require("./logger");
const yargs = require("yargs");
const start = (argv) => __awaiter(void 0, void 0, void 0, function* () {
    let ganache;
    if (argv.run && argv.run === "docker")
        ganache = yield docker_1.startDocker("ganache", argv);
    if (argv.run && argv.run === "process")
        ganache = yield ganache_1.startGanache(argv);
    if (ganache) {
        deploy_1.migrate().then(() => {
            console.log("Your clone of mainnet is ready to be used at http://localhost:8545 🚀");
        });
        let interval;
        if (argv.node && argv.node === "full")
            interval = setInterval(() => __awaiter(void 0, void 0, void 0, function* () {
                console.log("Reloading full node:\n");
                let reset;
                if (argv.run && argv.run === "docker")
                    reset = yield docker_1.restartContainer(ganache);
                if (argv.run && argv.run === "process")
                    reset = yield ganache_1.restartGanache(ganache, argv);
                if (reset) {
                    ganache = reset;
                    try {
                        yield deploy_1.migrate();
                    }
                    catch (e) {
                        throw new Error(`Failed to seed while refreshing\n${e}`);
                    }
                }
            }), 30 * 60 * 1000);
        process.once("SIGINT", () => {
            console.log("\n", "Stopping cloned blockchain...", "\n");
            if (interval)
                clearInterval(interval);
            if (argv.run && argv.run === "docker") {
                ganache.stop(function (err, data) {
                    console.log("\n", "Removing cloned blockchain...", "\n");
                    if (!ganache)
                        return;
                    ganache.remove(function (err, data) {
                        process.exit(0);
                    });
                });
            }
        });
        process.once("SIGTERM", () => {
            console.log("\n", "Stopping cloned blockchain...", "\n");
            if (interval)
                clearInterval(interval);
            if (argv.run && argv.run === "docker") {
                ganache.stop(function (err, data) {
                    console.log("\n", "Removing cloned blockchain...", "\n");
                    if (!ganache)
                        return;
                    ganache.remove(function (err, data) {
                        process.exit(0);
                    });
                });
            }
        });
    }
});
const main = (argv = yargs.argv) => __awaiter(void 0, void 0, void 0, function* () {
    process.once("SIGINT", () => {
        console.log("\n");
    });
    process.once("SIGTERM", () => {
        console.log("\n");
    });
    try {
        const config = (yield config_1.getConfig("cwa-config.js")) ||
            (yield config_1.getConfig("cwa-config.json"));
        const load = yield config_1.loadConfig(config);
        Object.entries(load).forEach(([key, value]) => (argv[key] = value));
        yield args_1.promptMissingArgs(argv);
        yield start(argv);
    }
    catch (error) {
        logger_1.logError(error);
        process.exit(1);
    }
});
exports.default = main;
//# sourceMappingURL=clone.js.map