"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.migrate = exports.fundERC20 = exports.buildFundTransactions = void 0;
const util = require("util");
const setTimeoutPromise = util.promisify(setTimeout);
const exec = util.promisify(require("child_process").exec);
const spawn = require("child_process").spawn;
const chalk = require("chalk");
const constants_1 = require("./constants");
const Web3 = require("web3");
const money_legos_1 = require("@studydefi/money-legos");
const logger_1 = require("./logger");
const config_1 = require("./config");
function hasLocalDependency(packageName) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            yield exec(`npm ls ${packageName}`);
            return true;
        }
        catch (e) {
        }
    });
}
function hasGlobalDependency(packageName) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            yield exec(`npm ls -g ${packageName}`);
            return true;
        }
        catch (e) {
            return false;
        }
    });
}
function buildFundTransactions(tokenName, liqudityAddress, tokenAmount = 200, accountsCount = 1) {
    if (!tokenName)
        throw new Error("ERROR: Must given a token name to transfer");
    if (!liqudityAddress || !Web3.utils.isAddress(liqudityAddress))
        throw new Error("ERROR: Must give a liquidity provider address to transfer from");
    if (!money_legos_1.legos.erc20[tokenName] || !constants_1.LIQUIDITY[tokenName])
        throw new Error(`Unable not seed ${tokenName}`);
    return Array(accountsCount)
        .fill(1)
        .map((_, i) => new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
        try {
            const web3 = new Web3(constants_1.URI);
            const accounts = yield web3.eth.getAccounts();
            const abi = money_legos_1.legos.erc20[tokenName].abi;
            const contract = new web3.eth.Contract(abi, money_legos_1.legos.erc20[tokenName].address);
            yield contract.methods
                .transfer(accounts[i], tokenAmount)
                .send({ from: constants_1.LIQUIDITY[tokenName] })
                .then(function (result) {
                resolve(result);
            });
        }
        catch (e) {
            return reject(e);
        }
    })));
}
exports.buildFundTransactions = buildFundTransactions;
function fundERC20(logger) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve, reject) => {
            logger.log("Seeding ERC20 tokens...");
            setTimeoutPromise(5000).then(() => __awaiter(this, void 0, void 0, function* () {
                try {
                    const seed = yield Promise.all(Object.entries(constants_1.LIQUIDITY).reduce((acc, [token, address]) => {
                        try {
                            const transactions = buildFundTransactions(token, address);
                            return [...acc, ...transactions];
                        }
                        catch (e) {
                            console.error(e);
                            return acc;
                        }
                    }, []));
                    if (seed) {
                        logger.succeed(`Seeding ERC20 tokens... ${chalk.green("success!")}`, true);
                        resolve(true);
                    }
                }
                catch (e) {
                    logger.fail(`Seeding ERC20 tokens... ${chalk.red("failed")}`, true);
                    reject(false);
                }
            }));
        });
    });
}
exports.fundERC20 = fundERC20;
function migrate() {
    return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
        const spinner = new logger_1.Logger();
        try {
            yield fundERC20(spinner);
        }
        catch (error) {
            throw `Error while running deployments:\n${error}`;
        }
        try {
            const truffleConfig = yield config_1.getConfig("truffle-config.js");
            if (truffleConfig) {
                spinner.log(chalk.green("Truffle project found"));
                const installed = hasGlobalDependency("truffle") || hasLocalDependency("truffle");
                if (installed) {
                    spinner.log("Deploying smart contracts...");
                    const run = spawn("truffle", ["migrate", "--network", "develop"]);
                    run.stdout.on("data", function (data) {
                        spinner.log("Deploying smart contracts..." + data.toString(), true);
                    });
                    run.on("exit", function (code) {
                        spinner.succeed(`Deploying smart contracts... ${chalk.green("success!")}`);
                        return resolve();
                    });
                }
            }
            const buidler = yield config_1.getConfig("buidler.config.js");
            if (buidler) {
                spinner.log(chalk.green("Buidler project found"));
                const installed = hasGlobalDependency("@nomiclabs/buidler") ||
                    hasLocalDependency("@nomiclabs/buidler");
                if (installed) {
                    spinner.log("Deploying smart contracts...");
                    spinner.log("Deploying smart contracts...\n    Compiling");
                    yield exec(`npx buidler compile`, (error, stdout, stderr) => {
                        if (error) {
                            console.error(`exec error: ${error}`);
                            return resolve();
                        }
                        console.log(`stdout: ${stdout}`);
                        console.error(`stderr: ${stderr}`);
                    });
                    const run = spawn("npx", [
                        "buidler",
                        "run",
                        "--network",
                        "development",
                        "scripts/deploy.js",
                    ]);
                    run.stdout.on("data", function (data) {
                        spinner.log("Deploying smart contracts..." + data.toString());
                    });
                    run.on("exit", function (code) {
                        spinner.succeed(`Deploying smart contracts... ${chalk.green("success!")}`);
                    });
                }
            }
            if (!buidler && !truffleConfig) {
                spinner.stop();
                return resolve();
            }
        }
        catch (error) {
            spinner.fail(`Deploying smart contracts... ${chalk.red("failed")}`);
            throw `Error while running deployments:\n${error}`;
        }
    }));
}
exports.migrate = migrate;
//# sourceMappingURL=deploy.js.map