"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.restartContainer = exports.startDocker = exports.isRunning = exports.isInstalled = void 0;
const constants = require("./constants");
function isInstalled() {
    return __awaiter(this, void 0, void 0, function* () {
        const { exec } = yield Promise.resolve().then(() => require("child_process"));
        return new Promise((resolve) => {
            exec("which docker", (error) => resolve(!error));
        });
    });
}
exports.isInstalled = isInstalled;
function isRunning(docker) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const result = yield docker.ping();
            return result.toString() === "OK";
        }
        catch (error) {
            return false;
        }
    });
}
exports.isRunning = isRunning;
function startDocker(container, argv) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
            try {
                const Docker = yield Promise.resolve().then(() => require("dockerode"));
                if (!(yield isInstalled()))
                    return reject("Docker is not installed!");
                const docker = new Docker({ socketPath: "/var/run/docker.sock" });
                if (!(yield isRunning(docker)))
                    return reject("Docker is not running, please start it!");
                console.log("\n", "Pulling latest image", "\n");
                const containerConfig = constants.CONTAINERS[container](argv);
                yield docker.pull(containerConfig.Image, {});
                yield docker.createContainer(containerConfig, (err, container) => {
                    container.start((err, data) => __awaiter(this, void 0, void 0, function* () {
                        container.attach({ stream: true, stdout: true, stderr: true }, function (err, stream) {
                            stream.pipe(process.stdout, { end: false });
                        });
                        resolve(container);
                    }));
                });
            }
            catch (error) {
                console.error(111, error);
                reject(error);
            }
        }));
    });
}
exports.startDocker = startDocker;
function restartContainer(container) {
    return new Promise((resolve, reject) => {
        try {
            console.log("restarting...");
            container.stop(() => {
                container.start(() => {
                    resolve(true);
                });
            });
        }
        catch (e) {
            console.error("Failed to restart container", e);
        }
    });
}
exports.restartContainer = restartContainer;
//# sourceMappingURL=docker.js.map