"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Logger = exports.logError = void 0;
const chalk = require("chalk");
const ora = require("ora");
const clog = console.log;
function logError(message) {
    clog(`\n${chalk.red(message)}\n`);
}
exports.logError = logError;
const oraOptions = { spinner: "squareCorners" };
class Logger {
    constructor(verbose = false) {
        this.verbose = verbose;
        if (!verbose) {
            const logger = ora(oraOptions);
            this.logger = logger.start();
        }
    }
    clean(message) {
        const split = message.split("\n");
        const text = split.find((chunk) => {
            return chunk && /^[a-zA-Z0-9_.\s]*$/.test(chunk);
        });
        return text;
    }
    log(message, clean) {
        if (this.verbose)
            return this.vlogger.log(message);
        let text = message.slice();
        if (clean)
            text = this.clean(text);
        if (text)
            this.logger.text = message;
    }
    succeed(message, continueSpinning) {
        if (this.verbose)
            return ora(oraOptions).succeed(message);
        this.logger.succeed(message);
        if (continueSpinning)
            this.logger = ora(oraOptions).start();
    }
    fail(message, continueSpinning) {
        if (this.verbose)
            return ora(oraOptions).fail(message);
        this.logger.fail(message);
        if (continueSpinning)
            this.logger = ora(oraOptions).start();
    }
    start() {
        if (this.logger && !this.logger.isSpinning)
            this.logger.start();
    }
    stop() {
        if (this.logger && this.logger.isSpinning)
            this.logger.stop();
    }
}
exports.Logger = Logger;
//# sourceMappingURL=logger.js.map